#!/bin/bash
#
#  LoRMA
#
#  Copyright (C) 2016 Riku Walve, Leena Salmela
#
#  Contact: leena.salmela@cs.helsinki.fi
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Affero General Public License as
#  published by the Free Software Foundation, either version 3 of the
#  License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Affero General Public License for more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
###########################################################################
#
# Runs LoRDEC iterations and LoRMA step
#

# Directory where this script is saved
DIR=$(cd "$(dirname "${BASH_SOURCE[0]}" )" && pwd)

# Directory where LoRDEC is installed
LORDECDIR=$DIR

# LoRDEC binaries
LORDEC=$LORDECDIR/lordec-correct
TRIMSPLIT=$LORDECDIR/lordec-trim-split

#LoRMA binary
LORMA=$DIR


usage() {
  echo -e "Usage: $0 [-s] [-n] [-start <19> -end <61> -step <21> -threads <6> -friends <7> -k <19>] *.fasta" 1>&2
  echo -e "\t-s saves the sequence data of intermediate LoRDEC steps" 1>&2
  echo -e "\t-n skips LoRDEC steps" 1>&2
  exit 1
}

# set defaults
START_K=19
END_K=61
K_STEP=21

FRIENDS=7
THREADS=6
LORMA_K=19

SKIP=0
SAVE=0
FILE=''

while true; do
  case "$1" in
    -s)
      shift
      SAVE=1 ;;

    -n)
      shift
      SKIP=1 ;;

    -start)
      shift
      case "$1" in
        *[!0-9]* | "") ;;
        *) START_K="$1"; shift ;;
      esac ;;

    -end)
      shift
      case "$1" in
        *[!0-9]* | "") ;;
        *) END_K="$1"; shift ;;
      esac ;;

    -step)
      shift
      case "$1" in
        *[!0-9]* | "") ;;
        *) K_STEP="$1"; shift ;;
      esac ;;

    -threads)
      shift
      case "$1" in
        *[!0-9]* | "") ;;
        *) THREADS="$1"; shift ;;
      esac ;;

    -friends)
      shift
      case "$1" in
        *[!0-9]* | "") ;;
        *) FRIENDS="$1"; shift ;;
      esac ;;

    -k)
      shift
      case "$1" in
        *[!0-9]* | "") ;;
        *) LORMA_K="$1"; shift ;;
      esac ;;

    -*)
      echo "$0: Unrecognized option "$1"" >&2
      usage ;;

    *)
      if [[ "$1" = '' ]]; then
        break
      fi

      if [[ $FILE = '' ]]; then
        FILE="$1"
      else
        echo "$0: Unrecognized option "$1"" >&2
        usage
      fi
      shift ;;
  esac
done

if [[ $FILE = '' ]]; then
  usage
fi

READS=$FILE
LAST=$READS

if [ $SKIP -eq 0 ]; then
  K=$START_K
  while [ $K -le $END_K ]; do
    READS=reads-k"$K".fasta

    $LORDEC -c -s 4 -k $K -i $LAST -2 $LAST -o $READS 2> lordec-"$K"-stderr.log 1> lordec-"$K"-stdout.log

    if [ $SAVE -eq 0 ] && [ $LAST != $FILE ]; then
      rm $LAST *.h5
    fi

    LAST=$READS
    K=$(($K+$K_STEP))
  done
fi

$TRIMSPLIT -i $READS -o trim.fasta

if [ $SAVE = 0 ]; then
  rm $READS
fi

READS=trim.fasta

$LORMA -k $LORMA_K -friends $FRIENDS -nb-cores $THREADS -reads $READS -output final.fasta -discarded discarded.fasta

if [ $SAVE = 0 ]; then
 rm trim.fasta discarded.fasta
fi
